function [vert, optmesh, L,M] = convomesh(n,a,b,C)

% computes an optimal polynomial mesh on a planar C^2 convex domain,
% given a Lipschitz continuous parametrization of the boundary by the 
% function sigma below; the convex domain is considered as a starlike 
% domain with respect to one of its points 

% by Federico Piazzon and Marco Vianello, University of Padova
% November 29, 2012


% INPUT: 

% n = polynomial degree 

% a,b = enpoints of the parameter interval 

% C = 2-component vector of the star center

% OUTPUT: 
% vert = discretized boundary 

% optmesh = 2-column array of the mesh points

% L = length of the boundary

% M = maximal curvature of the boundary



% FUNCTION BODY

NN=10; tol=10^(-6); err=tol+1; alpha=0.5;

% boundary length by polygonal approximation
Lold=0;
while (err > tol) 
NN=2*NN;
t=linspace(a,b,NN);h=(b-a)/NN;
vert=sigma(t);
diff=vert(2:end,:)-vert(1:end-1,:);
delta=sqrt(diff(:,1).^2+diff(:,2).^2);
Lnew=sum(delta);
err=abs(Lnew-Lold);
Lold=Lnew;
end;

% L is the approximate length
L=Lnew;

% approximate unit tangent vectors
newvert=[vert(end-1,:); vert; vert(2,:)];
chord=(newvert(3:end,:)-newvert(1:end-2,:));
utang(:,1)=chord(:,1)./sqrt((chord(:,1).^2+chord(:,2).^2));
utang(:,2)=chord(:,2)./sqrt((chord(:,1).^2+chord(:,2).^2));
% approximate curvatures
delta1=utang(2:end,:)-utang(1:end-1,:);
curv=sqrt(delta1(:,1).^2+delta1(:,2).^2)./delta;

% computing the boundary mesh 
M=max(curv);
N=ceil(M*n*L/(2*alpha)); 
deltas=L/N;go=true;j=1;k=1;
while (go==true) 
partL=0;
while(partL<deltas & go==true)
partL=partL+delta(j);
j=j+1;if j>length(delta) go=false; end;
end;
if partL>deltas 
j=j-1;
end;
mesh(k,:)=vert(j,:);k=k+1;
end;

% radial Chebyshev-Lobatto points
ind=(0:2*n);
xi=cos(ind*pi/(2*n));

% computing the optimal mesh and plotting 
C1=[C(1)*ones(length(vert),1) C(2)*ones(length(vert),1)];
C2=[C(1)*ones(length(mesh),1) C(2)*ones(length(mesh),1)];
optmesh=[ ];
for i=1:2*n+1
% scaling the boundary points 
vv=(1+xi(i))*(vert-C1)/2;
mm=(1+xi(i))*(mesh-C2)/2;
optmesh=[optmesh;mm];
% plot(vv(:,1),vv(:,2))
% hold on
% axis square
% plot(mm(:,1),mm(:,2),'.','MarkerSize',8);
end;

end



function vert = sigma(t)

% EXAMPLE: convex limacon
s=1;
a1=s;a2=2.05*s;
vert(:,1)=(a2+a1*cos(t)).*cos(t);
vert(:,2)=(a2+a1*cos(t)).*sin(t);

end

