clear all
close all
%-------------------------------------------------------------------
% This is the main file that produces a 3-dimensional wam for 
% a (generalized) cone, extracts approximate Fekete or Discrete
% Leja points, computes the Lebesgue constants and the least-squares
% norm
%
% by Stefano De Marchi and Marco Vianello, University of Padova 
% 
% January 2013
%
%--------------------------------------------------------------------
nmax=4;      % max polynomial degree
type=1;      % mesh type (values from 1 to 7)

Vert=[0,0,1];% For the cylinder Vert(3) should be inf or -inf
ratio=1;     % if less than 1 gives a part of the cone
leja=0;      % computes approximate Fekete points
             % leja=1: computes approximate Leja points

for n=1:nmax

switch type
    case 1 
        P=pdpts(n);    % 2d wam for the square (Padua points)
        P1=pdpts(2*n);
        pcl=[-1 -1; -1 1; 1 1; 1 -1; -1 -1];
    case 2
        P=wamdisk(n);   % 2d wam for the disk
        P1=wamdisk(2*n);
        t=linspace(0,2*pi,200)';
        pcl=[cos(t) sin(t)];
    case 3
        if nmax <= 10
          P=set_disk_leb(n); %lebesgue pts on the disk
          P1=set_disk_leb(2*n);
          t=linspace(0,2*pi,200)';
          pcl=[cos(t) sin(t)];
        else
          error('choose a smaller n ');
        end
    case 4
        [MM, P] = convomesh(n,0,2*pi,[0 0]); %wam on a planar C^2 convex domain
        [MM1, P1] = convomesh(2*n,0,2*pi,[0 0]);
         pcl=MM;
    case 5
        sides=10; % number of sides for the regular polygon
        [PP,pcl]=meshpolyg(2*n,sides);
        P = approxfek(2*n,PP,0,[],[]);
        P1 =meshpolyg(3*n,sides);
    case 6 
        S=[0 0; 1 0; 1 1/2; 0.2 1];
        P=wamquadrangle(n,S);   %quadrangle with vertices S
        P1=wamquadrangle(2*n,S);
        pcl=[S; S(1,:)];
    case 7
       cheb_lob=cos([0:n]*pi/n);   % Chebyshev-Lobatto grid 
      [Px,Py]=ndgrid(cheb_lob);
       P=[Px(:) Py(:)];
       cheb_lob=cos([0:2*n]*pi/n);    
       [Px,Py]=ndgrid(cheb_lob);
       P1=[Px(:) Py(:)];
       pcl=[-1 -1; -1 1; 1 1; 1 -1; -1 -1];
    otherwise
        error('Invalid mesh type ! Mesh type between 1 to 7 ');
end    

% compute the 3-dimensional wams for cones
M = wamcone(P,Vert,n,ratio);
M1 = wamcone(P1,Vert,n,ratio);


% extracts approximate Fekete or Leja interpolation points from a 2d
% or 3d mesh/cloud and estimates their Lebesgue constant
% and the norm of the LS projection operator on the mesh
[fekete,T,leb(n),nls(n)] = approxfek1(n,M,leja,M1,[]);

clear T;
end

 dim=size(P,1);
 M0=[P zeros(dim,1)];
 M10=M(1:dim,:);

%  ind=find(M==-1);
%  line=[M(ind(1),:,:); Vert];
%  
%  ind1=find(M==Vert(3));
%  line1=[M(ind(1),:,:); M(ind1(1),:,:)];
 
 
 % plot the mesh and the Fekete points
 plot3(M(:,1),M(:,2),M(:,3),'*','MarkerSize',5);
 hold on
 plot3(fekete(:,1),fekete(:,2),fekete(:,3),'ok')
 
 
 % plot the lines connecting the basis with the vertex
 for j=1:dim
  plot3([M0(j,1) M10(j,1)], [M0(j,2) M10(j,2)], [M0(j,3) M10(j,3)],'k','LineWidth',.5)
 end
 
% plot the boundary of the basis
plot(pcl(:,1), pcl(:,2),'b', 'MarkerSize',.2);


disp(' Lebesgue constants ' ); leb
disp(' Least-squares norm ' ); nls

hold off
axis off