clear all
close all
%-------------------------------------------------------------------
% This is the main file that produces a 3-dimensional wam for 
%(generalized) rotational domains; extracts approximate Fekete or discrete
% Leja points, computes the Lebesgue constants and the least-squares
% norm
%
% by Stefano De Marchi and Marco Vianello, University of Padova 
% 
% January 2013
%
%--------------------------------------------------------------------
format short 

nmax=4;       % max polynomial degree
type=2;       % mesh type (values from 1 to 7)

leja=0;       % computes approximate Fekete points
              % leja=1: computes approximate Leja points
r0=3;         % rotation radius length
z0=0;         % rotation center along z
omega=2/3*pi; % rotation angle

for n=1:nmax

switch type
    case 1 % 2d wam for the square (Padua points)
        P=pdpts(n);   
        P1=pdpts(2*n);
        pcl=[-1 -1; -1 1; 1 1; 1 -1; -1 -1];
    case 2 % 2d wam for the disk
        P=wamdisk(n);   
        P1=wamdisk(2*n);
        t=linspace(0,2*pi,200)';
        pcl=[cos(t) sin(t)];
    case 3 %lebesgue pts on the disk
        if nmax <= 10
          P=set_disk_leb(n);
          P1=set_disk_leb(2*n);
          t=linspace(0,2*pi,200)';
          pcl=[cos(t) sin(t)];
        else
          error('choose a smaller n ');
        end
    case 4 %wam on a planar C^2 convex domain
        [MM, P] = convomesh(n,0,2*pi,[0 0]); 
        [MM1, P1] = convomesh(2*n,0,2*pi,[0 0]);
        pcl=MM;
        clear MM MM1
    case 5 % regular polygon
        sides=10; 
        [PP,pcl]=meshpolyg(2*n,sides);
        P = approxfek(2*n,PP,0,[],[]);
        P1 =meshpolyg(3*n,sides);
    case 6 % quadrangle with vertices S
        S=[0 0; 1 0; 1 1/2; 0.2 1];
        P=wamquadrangle(n,S);   
        P1=wamquadrangle(2*n,S);
        pcl=[S; S(1,:)];
    case 7 % Chebyshev-Lobatto grid
       cheb_lob=cos([0:n]*pi/n);   
       [Px,Py]=ndgrid(cheb_lob);
       P=[Px(:) Py(:)];
       cheb_lob=cos([0:2*n]*pi/n);    
       [Px,Py]=ndgrid(cheb_lob);
       P1=[Px(:) Py(:)];
       pcl=[-1 -1; -1 1; 1 1; 1 -1; -1 -1];
    otherwise
        error('Invalid mesh type ! Mesh type between 1 to 7 ');
end    

% compute the 3-dimensional wams for rotational domains
[M, theta] = wamrot(P, n, r0, z0, omega);
[M1, theta] = wamrot(P1, n, r0, z0, omega);

% extract approximate Fekete or Leja interpolation points from a 2d
% or 3d mesh/cloud and estimates their Lebesgue constant
% and the norm of the LS projection operator on the mesh
[fekete, T, leb(n), nls(n)] = approxfek1(n, M, leja, M1, []);

clear T;
end

 
 figure
 plot3(M(:,1),M(:,2),M(:,3),'*','MarkerSize', 5);
 hold on
 plot3(fekete(:,1),fekete(:,2),fekete(:,3),'ok' );
 
 axis equal
 
 % plot arcs 
 
  dim=size(P,1);
  tt=angles(n,omega); 
  
  for i=1:dim
  for k=1:length(tt)-1
      tt1=linspace(tt(k),tt(k+1),200);
      PAx=(P(i,1)+r0)*cos(tt1);
      PAy=(P(i,1)+r0)*sin(tt1);
      PAz=(z0+P(i,2))*ones(1,length(tt1));
      plot3(PAx,PAy,PAz,'k');
  end
  end
  
 % plot circles on the torus
  
  tt2=linspace(0,2*pi,100);
  ss=sin(tt2); 
  cc=cos(tt2);
  P1=[cc' ss'];
 [Mc, theta] = wamrot(P1,n,r0,z0,omega);
 
  mm=length(cc);
  for j=1:length(theta)
    plot3(Mc((j-1)*mm+1:j*mm,1), Mc((j-1)*mm+1:j*mm,2), Mc((j-1)*mm+1:j*mm,3));
  end

  % plot the torus

  [x,y,z]=torus(1.0,200,r0);
  axis on
  colormap gray
  surf(x,y,z);
  alpha(0.3)
  shading flat
  
  axis tight


  disp(' Lebesgue constants ' ); leb
  disp(' Least-squares norm ' ); nls

  hold off
