function [V,index_j,index_k,index_l]=vandermonde_polygons(basis_type,n,wam,a,b,c,d,e,f)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OBIETTIVO:
% Costruzione della matrice di Vandermonde sulla WAM o sulla mesh di
% controllo
%
% INPUTS:
% basis_type=indica il tipo di base da utilizzare, Legendre (1), Chebyshev (2), Monomiale (3), Wade (4) 
% n=grado polinomiale
% wam=matrice che contiene le coordinate dei punti della WAM
% a=estremo inferiore dell'intervallo [a,b] lungo il primo asse su cui scalare la matrice
% b=estremo superiore dell'intervallo [a,b] lungo il primo asse su cui scalare la matrice
% c=estremo inferiore dell'intervallo [c,d] lungo il secondo asse su cui scalare la matrice
% d=estremo superiore dell'intervallo [c,d] lungo il secondo asse su cui scalare la matrice
% e=estremo inferiore dell'intervallo [e,f] lungo il terzo asse su cui scalare la matrice
% f=estremo superiore dell'intervallo [e,f] lungo il terzo asse su cui scalare la matrice
%
% OUTPUTS:
% V=matrice di Vandermonde sulla WAM
% index_j=vettore che contiene gli indici scelti relativi alla prima 
%         coordinataper la costruzione della base 
% index_k=vettore che contiene gli indici scelti relativi alla seconda 
%         coordinata per la costruzione della base 
% index_l=vettore che contiene gli indici scelti relativi alla terza 
%         coordinata per la costruzione della base 
%--------------------------------------------------------------------------
%% Copyright (C) 2010 Martina Marchioro, Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Martina Marchioro <martina.marchioro@gmail.com>
%%          Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: September 10, 2010.
%--------------------------------------------------------------------------
V=[];

%Individuiamo le combinazioni degli indici minori uguali di n
polynomial_degree=n;
j = 0:n;
[j1,j2,j3]= meshgrid(j);
k = find(j1(:)+j2(:)+j3(:)<n+1); %considero tutte le combinazioni possibili dei 
                                 %"gradi" dei vari polinomi.
polynomial_indexes = [j1(k) j2(k) j3(k)];%considero solamente quelle combinazioni 
                                    %che hanno solamente grado minore uguale ad n.

% Punti x y z
pts=wam;
x=pts(:,1); 
y=pts(:,2); 
z=pts(:,3);
elements_in_basis=size(polynomial_indexes,1);

% Calcolo della matrice di Vandermonde con una variabile, otteniamo le
% matrici di Vandermonde Vx Vy Vz
switch basis_type
    case  2
        s=ones(length(x),1)*([0:n]); %matrice che contiene il valore k del T_k
                                      %da moltiplicare all'arcocoseno
        xx=(2*x-a-b)/(b-a); 
        yy=(2*y-c-d)/(d-c);
        zz=(2*z-e-f)/(f-e);
        argx=xx*ones(1,n+1);%argomento dell'arcocoseno x
        argy=yy*ones(1,n+1);%argomento dell'arcocoseno y
        argz=zz*ones(1,n+1);%argomento dell'arcocoseno z
        Vx=cos(s.*acos(argx)); 
        Vy=cos(s.*acos(argy));
        Vz=cos(s.*acos(argz));
          
     case  3
        x=(x-(a+b)/2)*(2/(b-a)); 
        y=(y-(c+d)/2)*(2/(d-c)); 
        z=(z-(e+f)/2)*(2/(f-e)); 
        e=ones(length(x),1)*[0:n]; 
        xx=x*ones(1,n+1);
        yy=y*ones(1,n+1);
        zz=z*ones(1,n+1);
        Vx=xx.^e; %monomiale in x
        Vy=yy.^e; %monomiale in y
        Vz=zz.^e; %monomiale in z
         
     otherwise
        Vx=vandermonde_1D(x,polynomial_degree,a,b);
        Vy=vandermonde_1D(y,polynomial_degree,c,d);
        Vz=vandermonde_1D(z,polynomial_degree,e,f);
end

% A partire da Vx Vy Vz costruzione della matrice di Vandermonde nelle tre
% variabili
index_j=polynomial_indexes(:,1);
index_k=polynomial_indexes(:,2);
index_l=polynomial_indexes(:,3);
V=((Vx(:,(index_j)+1).*Vy(:,(index_k)+1)).*Vz(:,(index_l)+1));

clear j j1 j2 j3 wam x y z Vx Vy Vz  n basis_type coef xx yy zz argx argy argz;
return
